"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _THead = require("./THead");

var _Tr = require("./Tr");

var _TBody = _interopRequireDefault(require("./TBody"));

var _TFoot = _interopRequireDefault(require("./TFoot"));

var _Header = _interopRequireDefault(require("../Header"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * ProTable Options
 * 
 * @typedef { Object } ProTable.Options
 * @property { string[] } classes - Classes name of the table
 * @property { Object } thead - Thead
 * @property { Object } tbody - TBody
 * @property { Object } columns - Custom columns of table
 * @property { Number } limit - page limit
 * @property { Object } pagination - Pagination specific options
 * @property { string } pagination.type - Pagination type
 * @property { Object } pagination.rowsPerPage - Pagination rows per page options
 * @property { string } pagination.rowsPerPage.text - Pagination rows per page text
 * @property { string[] } pagination.rowsPerPage.selectClasses
 * @property { Number[] } pagination.rowsPerPage.ranges
 * @property { Object } search
 * @property { string } search.placeholder
 * @property { string[] } search.classes
 * @property { string } search.notFoundText
 */

/**
 * ProTable class
 *
 * @class ProTable
 * @property { ProTable.Options } options
 */
var ProTable = /*#__PURE__*/function () {
  /**
   * Creates an instance of ProTable.
   * @param { string } selector - DOM selector where the ProTable should replace
   * @param { ProTable.Options } options
   * @memberof ProTable
   */
  function ProTable(selector, options) {
    _classCallCheck(this, ProTable);

    this.selector = selector;
    /**
     * @type ProTable.Options
     */

    var defaultOptions = {
      classes: [],
      thead: {},
      limit: 10,
      page: 1,
      keyword: null,
      pagination: {
        type: 'default'
      },
      order: {
        key: null,
        direction: null
      }
    };
    /**
     * @type { ProTable.Options }
     */

    this.options = _objectSpread(_objectSpread({}, defaultOptions), options);
    this.thead = null;
    this.tbody = null;
    this.$dom = document.createElement('section');
    this.$dom.classList.add('protable');
  }

  _createClass(ProTable, [{
    key: "generateTable",
    value: function generateTable(_ref) {
      var columns = _ref.columns,
          rows = _ref.rows;
      this.$table = document.createElement('table');
      this.$dom.appendChild(this.$table);
      this.columns = this._formatColumns(columns);

      if (this.options.columns) {
        // merging
        for (var _key in this.options.columns) {
          var _col = this.options.columns[_key];

          if (Number.isInteger(_col.targetIndex)) {
            var _targetKey = Object.keys(this.columns)[_col.targetIndex];

            this.columns[_targetKey] = _objectSpread(_objectSpread({}, this.columns[_targetKey]), _col);
          } else if (this.columns[_key]) {
            this.columns[_key] = _objectSpread(_objectSpread({}, this.columns[_key]), _col);
          }
        }
      }

      this.rows = rows;

      this._generateHeader();

      this._generateThead();

      this._generateTbody();

      this._generateTFoot(); // apply options


      if (this.options.classes) {
        var _this$$table$classLis;

        (_this$$table$classLis = this.$table.classList).add.apply(_this$$table$classLis, _toConsumableArray(this.options.classes));
      }
    }
  }, {
    key: "_formatColumns",
    value: function _formatColumns(columns) {
      var _this = this;

      // if columns is Object, return as is
      if (columns !== null && columns.constructor === Object) {
        return columns;
      }

      var formatted = {};
      columns.forEach(function (_col) {
        if (_col !== null && _col.constructor === Object) {
          var key = Object.keys(_col)[0];
          formatted[key] = {
            label: "".concat(key[0].toUpperCase()).concat(key.substr(1)),
            childs: _this._formatColumns(Object.values(_col)[0])
          };
        } else {
          formatted[_col] = {
            label: "".concat(_col[0].toUpperCase()).concat(_col.substr(1))
          };
        }
      });
      return formatted;
    }
  }, {
    key: "_generateHeader",
    value: function _generateHeader() {
      this.header = new _Header["default"]({
        proTable: this
      });
      this.$dom.prepend(this.header.$dom);
    }
  }, {
    key: "_generateThead",
    value: function _generateThead() {
      this.thead = new _THead.THead({
        proTable: this,
        options: this.options.thead
      });
      this.$table.appendChild(this.thead.$dom);
    }
  }, {
    key: "_generateTbody",
    value: function _generateTbody() {
      if (this.tbody) {
        this.$table.removeChild(this.tbody.$dom);
      }

      this.tbody = new _TBody["default"]({
        proTable: this,
        options: this.options.tbody
      });
      this.$table.appendChild(this.tbody.$dom);
    }
  }, {
    key: "_generateTFoot",
    value: function _generateTFoot() {
      if (this.tfoot) {
        this.$table.removeChild(this.tfoot.$dom);
      }

      this.tfoot = new _TFoot["default"](this);
      this.$table.appendChild(this.tfoot.$dom);
    }
  }, {
    key: "setKeyword",
    value: function setKeyword(keyword) {
      this.options.keyword = keyword;
      this.setPage(1);
    }
  }, {
    key: "setPage",
    value: function setPage(page) {
      this.options.page = page;

      if (page < 1) {
        this.options.page = 1;
      }

      this.tbody.render();
      this.tfoot.render();
    }
  }, {
    key: "setLimit",
    value: function setLimit(limit) {
      this.options.page = 1;
      this.options.limit = parseInt(limit);
      this.tbody.render();
      this.tfoot.render();
    }
  }, {
    key: "setOrder",
    value: function setOrder(_ref2) {
      var key = _ref2.key,
          direction = _ref2.direction;
      this.options.order = {
        key: key,
        direction: direction
      };
      this.thead.render();
      this.setPage(1);
    }
  }, {
    key: "draw",
    value: function draw() {
      document.querySelector(this.selector).appendChild(this.$dom);
    }
  }]);

  return ProTable;
}();

var _default = ProTable;
exports["default"] = _default;