"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _Tr = _interopRequireDefault(require("./Tr"));

var _Td = _interopRequireDefault(require("./Td"));

var _ProTable = _interopRequireDefault(require("./ProTable"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * TBody Options
 * 
 * @typedef { Object } TBody.Options
 * @property { string[] } tdClasses - classes of each td
 */

/**
 * @class TBody
 * @param { TBody.Options } options
 */
var TBody = /*#__PURE__*/function () {
  /**
   * Creates an instance of TBody.
   * 
   * @param { Object } payload
   * @param { ProTable } payload.proTable
   * @param { TBody.Options } payload.options
   * @memberof TBody
   */
  function TBody(_ref) {
    var proTable = _ref.proTable,
        options = _ref.options;

    _classCallCheck(this, TBody);

    this.$dom = document.createElement('tbody');
    this.proTable = proTable;
    this.options = options;
    this.trs = this.generateTrs();
    this.render();
  }

  _createClass(TBody, [{
    key: "generateTrs",
    value: function generateTrs() {
      var _this = this;

      var columns = this.proTable.columns;
      var rows = this.proTable.rows;
      var trs = [];
      rows.forEach(function (_row) {
        var tr = new _Tr["default"]();
        tr.addTds(_this.generateTds(columns, _row));
        trs.push(tr);
      });
      return trs;
    }
    /**
     * Generate object array. The return of this function will next be
     * generated to object Tds
     * 
     * @param { Object } columns 
     * @param { Object[] } _row 
     */

  }, {
    key: "generateTds",
    value: function generateTds(columns, _row) {
      var tds = [];

      for (var _key in columns) {
        var _this$options;

        /**
         * the corresponding column of this row
         */
        var _col = columns[_key];
        /**
         * Row Content could be an array that consits with 2 items 
         *  - ['content', { classes: [...] }]
         *  - first item is the content
         *  - the second is the options
         * 
         * And could be just a string/Node
         */

        var rowContent = Array.isArray(_row[_key]) ? _row[_key][0] : _row[_key];
        var rowOptions = Array.isArray(_row[_key]) ? _row[_key][1] : {};

        if ((_this$options = this.options) === null || _this$options === void 0 ? void 0 : _this$options.tdClasses) {
          var _this$options2;

          rowOptions.classes = (rowOptions.classes || []).concat((_this$options2 = this.options) === null || _this$options2 === void 0 ? void 0 : _this$options2.tdClasses);
        } // override content if user defines targetIndex


        if (Number.isInteger(_col.targetIndex)) {
          rowContent = _row[Object.keys(this.proTable.columns)[_col.targetIndex]];
        } // override content if user defines contents callback


        if (this.proTable.options.contents && this.proTable.options.contents[_key]) {
          rowContent = this.proTable.options.contents[_key](rowContent); // check if the callback return is [content, { classes: [...] }]

          if (Array.isArray(rowContent) && rowContent.length > 1) {
            var _rowContent$;

            rowOptions.classes = (rowOptions.classes || []).concat(((_rowContent$ = rowContent[1]) === null || _rowContent$ === void 0 ? void 0 : _rowContent$.classes) || []);
            rowContent = rowContent[0];
          }
        }

        if (_col.childs) {
          // call generateTds recursively
          tds = tds.concat(this.generateTds(_col.childs, _row[_key]));
        } else {
          tds.push({
            key: _key,
            label: rowContent,
            options: _objectSpread(_objectSpread({}, rowOptions), {}, {
              searchable: typeof _col.searchable === 'boolean' ? !!_col.searchable : true
            })
          });
        }
      } // console.log('generateTds', tds)


      return tds;
    }
    /**
     * rendering trs
     *
     * @memberof TBody
     */

  }, {
    key: "render",
    value: function render() {
      var _this2 = this;

      // delete rendered trs
      while (this.$dom.lastChild) {
        this.$dom.removeChild(this.$dom.lastChild);
      }

      var limit = this.proTable.options.limit;
      var page = this.proTable.options.page;
      var start = (page - 1) * limit;
      this.filteredTrs.sort(this._sort.bind(this)).slice(start, this.proTable.options.limit * page).forEach(function (_tr) {
        return _this2.$dom.appendChild(_tr.$dom);
      }); // if there is no result for given keyword

      if (this.filteredTrs.length < 1 && this.proTable.options.keyword) {
        var _this$proTable$option;

        var tr = new _Tr["default"]();
        var td = new _Td["default"]({
          child: ((_this$proTable$option = this.proTable.options.search) === null || _this$proTable$option === void 0 ? void 0 : _this$proTable$option.notFoundText) || 'Not matching records found',
          options: {
            attrs: {
              colspan: this.proTable.thead.columnsCount
            },
            style: {
              textAlign: 'center'
            }
          }
        });
        tr.pushTd(td);
        this.$dom.appendChild(tr.$dom);
      }
    }
  }, {
    key: "_filter",

    /**
     * Filter by Tr
     *
     * @param { Tr } _tr
     * @memberof TBody
     */
    value: function _filter(_tr) {
      var keyword = this.proTable.options.keyword || '';
      return !!_tr.childs.find(function (_child) {
        var _child$$dom, _child$$dom$innerText, _child$$dom$innerText2;

        return _child.options.searchable && (_child === null || _child === void 0 ? void 0 : (_child$$dom = _child.$dom) === null || _child$$dom === void 0 ? void 0 : (_child$$dom$innerText = _child$$dom.innerText) === null || _child$$dom$innerText === void 0 ? void 0 : (_child$$dom$innerText2 = _child$$dom$innerText.toLowerCase()) === null || _child$$dom$innerText2 === void 0 ? void 0 : _child$$dom$innerText2.indexOf(keyword.toLowerCase())) > -1;
      });
    }
  }, {
    key: "_sort",
    value: function _sort(a, b) {
      var order = this.proTable.options.order;

      if (order.key) {
        var tdA = a.childs.find(function (_td) {
          return _td.key === order.key;
        });
        var tdB = b.childs.find(function (_td) {
          return _td.key === order.key;
        }); // check if the value is valid number or not

        var contentA = !isNaN(tdA.$dom.innerText) ? parseFloat(tdA.$dom.innerText) : tdA.$dom.innerText;
        var contentB = !isNaN(tdB.$dom.innerText) ? parseFloat(tdB.$dom.innerText) : tdB.$dom.innerText; // ascending

        if (order.direction === 'asc') {
          return contentA > contentB ? 1 : -1;
        } else {
          return contentA > contentB ? -1 : 1;
        }
      }

      return 0;
    }
  }, {
    key: "filteredTrs",
    get: function get() {
      return this.trs.filter(this._filter.bind(this));
    }
  }]);

  return TBody;
}();

var _default = TBody;
exports["default"] = _default;