"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _ProTable = _interopRequireDefault(require("../components/table/ProTable"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * @typedef { Object } FromArray.Data
 * @property { string[]|Object } columns
 * @property { Object[] } rows
 */

/**
 * Helper to create instance of ProTable using array data
 * 
 * @param { string | Node } elId 
 * @param { array | FromArray.Data } data 
 * @param { ProTable.Options } options 
 */
var fromArray = function fromArray(elId, data, options) {
  var columns = data.columns || generateColumns(data);
  var proTable = new _ProTable["default"](elId, options);
  proTable.generateTable({
    columns: columns,
    rows: data.rows || data
  });

  if (elId) {
    proTable.draw();
  }

  return proTable;
};
/**
 * Generate column from an array
 * 
 * @param [{ key: value }, { key: value }] array 
 * @returns { String[] }
 */


var generateColumns = function generateColumns(array) {
  var columns = [];
  var objectColumns = [];
  array.forEach(function (_item) {
    var _loop = function _loop(key) {
      if (!columns.find(function (_column) {
        return _column === key;
      })) {
        // check wheter the value is an object
        if (_item[key] !== null && _item[key].constructor === Object) {
          // call generateColumns recursively
          if (!objectColumns.find(function (_objectColumn) {
            return _objectColumn === key;
          })) {
            objectColumns.push(key);
            columns.push(_defineProperty({}, key, generateColumns([_item[key]])));
          }
        } else {
          columns.push(key);
        }
      }
    };

    for (var key in _item) {
      _loop(key);
    }
  });
  return columns;
};

var _default = fromArray;
exports["default"] = _default;